library json_dynamic_form;

import 'package:flutter/material.dart';
import 'package:json_dynamic_form/models/Autogenerated.dart';
import 'package:json_dynamic_form/models/Values.dart';
import 'package:json_dynamic_form/utils/DynamicFormConstants.dart';

class JsonDynamicForm {
  final Function? setState;
  final List<dynamic>? data;
  late List<Autogenerated> _autogenerated;

  late List<Autogenerated> _controllers;
  List<Widget>? _widgets;

  ///Initialize the library with a mandatory [data] and [setState] method
  JsonDynamicForm({this.data, this.setState}) {
    _autogenerated = List<Autogenerated>.empty(growable: true);
    for (var item in data!) {
      _autogenerated.add(Autogenerated.fromJson(item));
    }
    _controllers =
        DynamicFormConstants().generateControllers(items: _autogenerated);
    _widgets = List<Widget>.empty(growable: true);
  }

  ///Generate supported widgets based on the JSON [data] provided
  List<Widget>? generateFields({bool firstTime = true}) {
    _widgets!.clear();
    for (Autogenerated item in _controllers) {
      //text field
      if (DynamicFormConstants.textTypes.contains(item.type)) {
        _generateTextField(item);
      }
      //radio buttons
      if (item.type == DynamicFormConstants.RADIO) {
        if (firstTime) {
          item.controller = item.values!.first.value;
        }
        _generateRadioButtons(item);
      }
      //dropdown button
      if (item.type == DynamicFormConstants.SELECT) {
        if (firstTime) {
          item.controller = item.values!.first.value;
        }
        _generateDropDown(item);
      }
      //checkbox
      if (item.values != null &&
          item.values!.isNotEmpty &&
          item.type == DynamicFormConstants.CHECKBOX) {
        _generateCheckbox(item);
      }
    }
    return _widgets;
  }

  _generateTextField(Autogenerated item) {
    _widgets!.add(TextField(
      controller: item.controller,
      decoration: InputDecoration(hintText: item.label),
    ));
  }

  _generateRadioButtons(Autogenerated item) {
    List<Widget> radioItems = List<Widget>.empty(growable: true);
    for (Values val in item.values!) {
      //Adding item to radio list
      radioItems.add(
        ListTile(
          title: Text(val.label!),
          leading: Radio<String?>(
            value: val.value,
            groupValue: item.controller.toString(),
            onChanged: (String? _value) {
              setState!(() {
                item.controller = _value.toString();
              });
            },
          ),
        ),
      );
    }
    _widgets!.add(Column(children: radioItems));
  }

  _generateDropDown(item) {
    _widgets!.add(DropdownButton(
        value: item.controller.toString(),
        onChanged: (String? value) {
          setState!(() {
            item.controller = value;
          });
        },
        items: item.values.map<DropdownMenuItem<String>>((Values value) {
          return DropdownMenuItem<String>(
            value: value.value,
            child: Text(value.label!),
          );
        }).toList()));
  }

  _generateCheckbox(Autogenerated item) {
    for (Values values in item.values!) {
      _widgets!.add(Row(
        children: [
          Text(values.label!),
          Checkbox(
            checkColor: Colors.white,
            value: values.controller,
            onChanged: (bool? value) {
              setState!(() {
                values.controller = value;
              });
            },
          )
        ],
      ));
    }
  }

  /// Return a List<dynamic> of the user inputs
  List<dynamic> printData() {
    List<dynamic> values = List<dynamic>.empty(growable: true);
    for (Autogenerated item in _controllers) {
      if (DynamicFormConstants.textTypes.contains(item.type)) {
        values
            .add({item.name: (item.controller as TextEditingController).text});
      }
      if (item.type == DynamicFormConstants.RADIO) {
        values.add({item.name: item.controller.toString()});
      }
      if (item.type == DynamicFormConstants.CHECKBOX) {
        List<String> checkItems = List<String>.empty(growable: true);
        for (Values value in item.values!) {
          checkItems.add(value.controller.toString());
        }
        values.add({item.name: checkItems});
      }
      if (item.type == DynamicFormConstants.SELECT) {
        values.add({item.name: item.controller.toString()});
      }
    }
    return values;
  }
}
